/*
** Module   : ImageGuiImpl.java
** Abstract : Implementation of a IMAGE widget for the GUI driver.
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140805 Created initial version.  Not yet fully tested/complete.
** 002 MAG 20140902 Use GUI driver system color API to get default colors for widgets drawing.
** 003 HC  20140911 GUI coordinate support - data types refactoring.
** 004 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 005 CA  20141021 Fixed image drawing and loading: tested only for the case when is used as a 
**                  window icon.
** 006 MAG 20141029 Added clipping rectangle when drawing. 
** 007 HC  20141021 Implemented window move/position support.
** 008 EVL 20141204 Adding image reloading capability.
** 009 EVL 20141208 Adding image widget initialization code.
** 010 EVL 20150108 Differentiate small icon from large one flag.  Rework for icon/image
**                  implementation.
*/

package com.goldencode.p2j.ui.client.gui;

import java.io.*;
import java.util.*;
import java.util.logging.*;

import javax.imageio.*;
import javax.imageio.stream.*;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.gui.driver.*;
import com.goldencode.p2j.util.*;

/**
 * Implementation of a IMAGE widget for the GUI driver.
 */
public class ImageGuiImpl
extends Image<GuiOutputManager>
{
   /** logger */
   protected static final Logger LOG = LogHelper.getLogger(ImageGuiImpl.class.getName());

   /** The image object. */
   protected java.awt.Image img;
   
   /** The graphics driver. */
   private GuiDriver gd;
   
   /** To keep track if this is the window icon or not. */
   private boolean isIcon = false;

   /**
    * Create a new image widget.
    * 
    * @param    id
    *           The widget's ID.
    */
   public ImageGuiImpl(WidgetId id)
   {
      super(id);

      this.gd = (GuiDriver) OutputManager.getDriver();
   }
   
   /**
    * Create a new image widget.
    * 
    * @param    id
    *           The widget's ID.
    * @param    isIcon
    *           Flag indicating the image is the window title bar icoon.
    */
   public ImageGuiImpl(WidgetId id, boolean isIcon)
   {
      this(id);

      this.isIcon = isIcon;
   }
   
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public ImageGuiImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (ImageConfig) cfg);
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, WidgetConfig cfg)
   {
      super.initialize(id, cfg);
      // screen driver
      gd = (GuiDriver) OutputManager.getDriver();
      // check if we need to preload image
      String imgSrcFile = ((ImageConfig) cfg).image;
      // filename exists, load it
      if (imgSrcFile != null && imgSrcFile.length() > 0)
      {
         setImageSource(imgSrcFile);
      }
   }

   /**
    * Draw this image
    */
   @Override
   public void draw()
   {
      
      NativePoint origin = physicalLocation();
      final NativeDimension d = physicalDimension();
      final NativeRectangle clip = clipRectangle(origin, d);
      
      gd.draw(origin, clip, new Runnable()
      {
         @Override
         public void run()
         {
            if (!drawImage())
            {
               gd.setColor(gd.getSysColor(SysColor.COLOR_WINDOW));
               gd.fillRect(0, 0, d.width, d.height);
            }
         }
      });
   }

   /**
    * Set the source of this image and load it.
    * 
    * @param    image
    *           The image source.
    */
   @Override
   public void setImageSource(String image)
   {
      super.setImageSource(image);
      
      loadImage();
   }

   /**
    * Force the source of this image and to be reloaded.
    * 
    * @param    image
    *           The image source.
    */
   public void replaceImageSource(String image)
   {
      if (image == null || image.equals(super.getImageSource()))
      {
         return;
      }
      
      // this causes the image to be replaced with the new one
      img = null;
      
      setImageSource(image);
   }

   /**
    * Get widget height.
    * 
    * @return  widget height.
    */
   @Override
   public double height()
   {
      int px = img == null ? 0 : img.getHeight(null);
      return screen().coordinates().heightFromNative(px);
   }

   /**
    * Get widget width.
    * 
    * @return  widget width.
    */
   @Override
   public double width()
   {
      int px = img == null ? 0 : img.getWidth(null);
      return screen().coordinates().widthFromNative(px);
   }

   /**
    * Draw this image.
    */
   protected boolean drawImage()
   {
      if (img != null)
      {
         CoordinatesConversion c = screen().coordinates();
         
         int w = c.widthToNative(width());
         int h = c.heightToNative(height());
         gd.drawImage(img, 0, 0, w, h);
      }
      
      return img != null;
   }

   /**
    * Load the image, from the server-side.
    */
   protected void loadImage()
   {
      String src = getImageSource();
      
      if (src != null)
      {
         byte[] byteImg = ThinClient.getInstance().getServer().loadImage(src, isIcon);

         if (byteImg == null)
         {
            img = null;
            return;
         }

         try
         {
            if (img == null)
            {
               img = ImageIO.read(new ByteArrayInputStream(byteImg));
            }
         }
         catch (IOException e)
         {
            if (LOG.isLoggable(Level.SEVERE))
            {
               LOG.log(Level.SEVERE, "Could not load image from source " + src, e);
            }

            img = null;
         }
      }
      else
      {
         // no image
         img = null;
      }
   }
}
